Ext.namespace('dSS', 'dSS.data');

dSS.data.ZoneStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for zone records
        var zoneRecord = Ext.data.Record.create([
        {
            name: 'name'
        },
        {
            name: 'id'
        },
        {
            name: 'primary',
            defaultValue: false
        },
        {
            name: 'unnamedDevices',
            defaultValue: false
        }
        ]);

        this.addEvents('zonedevnamechanged', 'zonedeviceleft');

        // a json reader to read the zone data
        var zoneReader = new Ext.data.JsonReader({
            root: 'zones'
        },
        zoneRecord);

        Ext.apply(this, {
            reader: zoneReader,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            }
        });
        dSS.data.ZoneStore.superclass.constructor.call(this, arguments);

        this.on('zonedeviceleft', function(zoneId) {
            if (zoneId === 0) {
                return;
            }
            var zone = this.getById(zoneId);
            if (zone === undefined) {
                return;
            }

            this.checkUnnamed(zone);
        },
        this);

        this.on('zonedevnamechanged', function(device) {
            var name = device.get('name');
            var zero = this.getById(0);
            var zone = this.getById(device.get('zoneId'));
            if (zone === undefined) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Could not find zone" +
                              " with id") + device.get('zoneId'));
                return;
            }

            if ((zero.get('unnamedDevices') === false) &&
                ((name === undefined) || (name.length === 0))) {
                zero.set('unnamedDevices', true);
                zero.commit();
            }

            // if all devices were named and one becomes unnamed we do not
            // need to cycle through all devies in zone but can tag the
            // zone right away
            if ((zone.get('unnamedDevices') === false) &&
                ((name === undefined) || (name.length === 0))) {
                zone.set('unnamedDevices', true);
                zone.commit();
                return;
            }

            this.checkUnnamed(zone, zero);
        },
        this);
    },

    checkUnnamed: function(zone, zero) {
        var unnamed = false;
        var records = dSS.data.Loader.getDeviceStore().query('zoneId',
                      zone.get('id'));
        records.each(
            function(r) {
                if ((r.get('name') === undefined) ||
                    (r.get('name').length === 0)) {
                    unnamed = true;
                }
            });

        if (unnamed !== zone.get('unnamedDevices')) {
            zone.set('unnamedDevices', unnamed);
            zone.commit();
        }

        if (zero !== undefined) {
            records = this.getRange(0);
            unnamed = false;
            var i;
            for (i = 0; i < records.length; i++) {
                if ((records[i].get('id') !== 0) &&
                     (records[i].get('unnamedDevices') === true)) {
                    unnamed = true;
                    break;
                }
            }

            if (unnamed !== zero.get('unnamedDevices')) {
                zero.set('unnamedDevices', unnamed);
                zero.commit();
            }
        }
    }
});

